@@ -185,7 +185,7 @@ int messages_set_notification_registration(void *session,
 			const struct messages_event *event, void *user_data),
 		void *user_data)
 {
-	return -EINVAL;
+	return -ENOSYS;
 }
 
 int messages_set_folder(void *s, const char *name, gboolean cdup)
@@ -310,7 +310,7 @@ int messages_get_messages_listing(void *session,
 				messages_get_messages_listing_cb callback,
 				void *user_data)
 {
-	return -EINVAL;
+	return -ENOSYS;
 }
 
 int messages_get_message(void *session, const char *handle,
@@ -318,13 +318,13 @@ int messages_get_message(void *session, const char *handle,
 				messages_get_message_cb callback,
 				void *user_data)
 {
-	return -EINVAL;
+	return -ENOSYS;
 }
 
 int messages_update_inbox(void *session, messages_update_inbox_cb callback,
 							void *user_data)
 {
-	return -EINVAL;
+	return -ENOSYS;
 }
 
 void messages_abort(void *session)
