@@ -36,10 +36,11 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <inttypes.h>
 
-#include <openobex/obex.h>
-#include <openobex/obex_const.h>
+#include <gobex.h>
 
+#include "obexd.h"
 #include "plugin.h"
 #include "log.h"
 #include "obex.h"
@@ -47,7 +48,7 @@
 #include "phonebook.h"
 #include "mimetype.h"
 #include "filesystem.h"
-#include "dbus.h"
+#include "manager.h"
 
 #define PHONEBOOK_TYPE		"x-bt/phonebook"
 #define VCARDLISTING_TYPE	"x-bt/vcard-listing"
@@ -160,8 +161,10 @@ static const uint8_t PBAP_TARGET[TARGET_SIZE] = {
 typedef int (*cache_entry_find_f) (const struct cache_entry *entry,
 			const char *value);
 
-static void cache_entry_free(struct cache_entry *entry)
+static void cache_entry_free(void *data)
 {
+	struct cache_entry *entry = data;
+
 	g_free(entry->id);
 	g_free(entry->name);
 	g_free(entry->sound);
@@ -222,8 +225,7 @@ static const char *cache_find(struct cache *cache, uint32_t handle)
 
 static void cache_clear(struct cache *cache)
 {
-	g_slist_foreach(cache->entries, (GFunc) cache_entry_free, NULL);
-	g_slist_free(cache->entries);
+	g_slist_free_full(cache->entries, cache_entry_free);
 	cache->entries = NULL;
 }
 
@@ -469,9 +471,12 @@ static int generate_response(void *user_data)
 	pbap->obj->buffer = g_string_new(VCARD_LISTING_BEGIN);
 	for (; l && max; l = l->next, max--) {
 		const struct cache_entry *entry = l->data;
+		char *escaped_name = g_markup_escape_text(entry->name, -1);
 
 		g_string_append_printf(pbap->obj->buffer,
-			VCARD_LISTING_ELEMENT, entry->handle, entry->name);
+			VCARD_LISTING_ELEMENT, entry->handle, escaped_name);
+
+		g_free(escaped_name);
 	}
 
 	pbap->obj->buffer = g_string_append(pbap->obj->buffer,
@@ -619,8 +624,7 @@ static void *pbap_connect(struct obex_session *os, int *err)
 	return pbap;
 }
 
-static int pbap_get(struct obex_session *os, obex_object_t *obj,
-							void *user_data)
+static int pbap_get(struct obex_session *os, void *user_data)
 {
 	struct pbap_session *pbap = user_data;
 	const char *type = obex_get_type(os);
@@ -636,9 +640,13 @@ static int pbap_get(struct obex_session *os, obex_object_t *obj,
 	if (type == NULL)
 		return -EBADR;
 
-	rsize = obex_aparam_read(os, obj, &buffer);
-	if (rsize < 0)
-		return -EBADR;
+	rsize = obex_get_apparam(os, &buffer);
+	if (rsize < 0) {
+		if (g_ascii_strcasecmp(type, VCARDENTRY_TYPE) != 0)
+			return -EBADR;
+
+		rsize = 0;
+	}
 
 	params = parse_aparam(buffer, rsize);
 	if (params == NULL)
@@ -651,14 +659,14 @@ static int pbap_get(struct obex_session *os, obex_object_t *obj,
 
 	pbap->params = params;
 
-	if (strcmp(type, PHONEBOOK_TYPE) == 0) {
+	if (g_ascii_strcasecmp(type, PHONEBOOK_TYPE) == 0) {
 		/* Always contains the absolute path */
 		if (g_path_is_absolute(name))
 			path = g_strdup(name);
 		else
 			path = g_build_filename("/", name, NULL);
 
-	} else if (strcmp(type, VCARDLISTING_TYPE) == 0) {
+	} else if (g_ascii_strcasecmp(type, VCARDLISTING_TYPE) == 0) {
 		/* Always relative */
 		if (!name || strlen(name) == 0)
 			/* Current folder */
@@ -667,7 +675,7 @@ static int pbap_get(struct obex_session *os, obex_object_t *obj,
 			/* Current folder + relative path */
 			path = g_build_filename(pbap->folder, name, NULL);
 
-	} else if (strcmp(type, VCARDENTRY_TYPE) == 0) {
+	} else if (g_ascii_strcasecmp(type, VCARDENTRY_TYPE) == 0) {
 		/* File name only */
 		path = g_strdup(name);
 	} else
@@ -683,16 +691,15 @@ static int pbap_get(struct obex_session *os, obex_object_t *obj,
 	return ret;
 }
 
-static int pbap_setpath(struct obex_session *os, obex_object_t *obj,
-		void *user_data)
+static int pbap_setpath(struct obex_session *os, void *user_data)
 {
 	struct pbap_session *pbap = user_data;
 	const char *name;
-	uint8_t *nonhdr;
+	const uint8_t *nonhdr;
 	char *fullname;
 	int err;
 
-	if (OBEX_ObjectGetNonHdrData(obj, &nonhdr) != 2) {
+	if (obex_get_non_header_data(os, &nonhdr) != 2) {
 		error("Set path failed: flag and constants not found!");
 		return -EBADMSG;
 	}
@@ -748,6 +755,7 @@ static struct obex_service_driver pbap = {
 	.name = "Phonebook Access server",
 	.service = OBEX_PBAP,
 	.channel = PBAP_CHANNEL,
+	.secure = TRUE,
 	.record = PBAP_RECORD,
 	.target = PBAP_TARGET,
 	.target_size = TARGET_SIZE,
@@ -967,7 +975,7 @@ static ssize_t vobject_pull_get_next_header(void *object, void *buf, size_t mtu,
 	if (!obj->buffer && !obj->aparams)
 		return -EAGAIN;
 
-	*hi = OBEX_HDR_APPARAM;
+	*hi = G_OBEX_HDR_APPARAM;
 
 	if (pbap->params->maxlistcount == 0 || obj->firstpacket) {
 		obj->firstpacket = FALSE;
@@ -987,11 +995,12 @@ static ssize_t vobject_pull_read(void *object, void *buf, size_t count)
 	DBG("buffer %p maxlistcount %d", obj->buffer,
 						pbap->params->maxlistcount);
 
-	if (!obj->buffer && !obj->aparams)
-		return -EAGAIN;
+	if (!obj->buffer) {
+		if (pbap->params->maxlistcount == 0)
+			return -ENOSTR;
 
-	if (pbap->params->maxlistcount == 0)
-		return -ENOSTR;
+		return -EAGAIN;
+	}
 
 	len = string_read(obj->buffer, buf, count);
 	if (len == 0 && !obj->lastpart) {
@@ -1019,7 +1028,7 @@ static ssize_t vobject_list_get_next_header(void *object, void *buf, size_t mtu,
 	if (!pbap->cache.valid)
 		return -EAGAIN;
 
-	*hi = OBEX_HDR_APPARAM;
+	*hi = G_OBEX_HDR_APPARAM;
 
 	if (pbap->params->maxlistcount == 0)
 		return array_read(obj->aparams, buf, mtu);
