@@ -34,13 +34,12 @@
 #include <signal.h>
 #include <fcntl.h>
 #include <termios.h>
-
-#include <openobex/obex.h>
-#include <openobex/obex_const.h>
+#include <inttypes.h>
 
 #include <glib.h>
 #include <gdbus.h>
 
+#include "obexd.h"
 #include "plugin.h"
 #include "server.h"
 #include "obex.h"
@@ -148,8 +147,8 @@ static int usb_connect(struct obex_server *server)
 	usb_io = g_io_channel_unix_new(fd);
 	g_io_channel_set_close_on_unref(usb_io, TRUE);
 
-	err = obex_server_new_connection(server, usb_io,
-					USB_TX_MTU, USB_RX_MTU);
+	err = obex_server_new_connection(server, usb_io, USB_TX_MTU,
+							USB_RX_MTU, TRUE);
 	if (err < 0)
 		goto failed;
 
@@ -241,11 +240,13 @@ static void *usb_start(struct obex_server *server, int *err)
 	if (dbus_connection_send_with_reply(connection,
 					msg, &call, -1) == FALSE) {
 		error("usb: unable to send mode_request");
+		dbus_message_unref(msg);
 		goto fail;
 	}
 
 	dbus_pending_call_set_notify(call, mode_request_reply, server, NULL);
 	dbus_pending_call_unref(call);
+	dbus_message_unref(msg);
 
 	id = g_dbus_add_signal_watch(connection, NULL, NULL,
 					"com.meego.usb_moded",
